<?php

namespace Drupal\simple_social_share\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Provides a configurable social sharing block.
 *
 * @Block(
 *   id = "simple_social_share_block",
 *   admin_label = @Translation("Simple Social Share Block"),
 *   category = @Translation("Social")
 * )
 */
class SimpleSocialShareBlock extends BlockBase {

  /**
   * Available social media platforms.
   */
  protected const AVAILABLE_PLATFORMS = [
    'facebook' => 'Facebook',
    'twitter' => 'Twitter',
    'linkedin' => 'LinkedIn',
    'whatsapp' => 'WhatsApp',
    'telegram' => 'Telegram',
    'pinterest' => 'Pinterest',
    'reddit' => 'Reddit',
    'tumblr' => 'Tumblr',
    'email' => 'Email',
  ];

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();

    $form['platforms'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Social Media Platforms'),
      '#description' => $this->t('Select which social media platforms to display.'),
    ];

    foreach (self::AVAILABLE_PLATFORMS as $key => $label) {
      $form['platforms'][$key] = [
        '#type' => 'checkbox',
        '#title' => $this->t($label),
        '#default_value' => $config['platforms-' . $key],
      ];
    }

    $form['show_copy_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Copy Link Button'),
      '#description' => $this->t('Display a button to copy the current page URL.'),
      '#default_value' => $config['show_copy_link'] ?? TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    foreach (self::AVAILABLE_PLATFORMS as $key => $label) {
      if ($form_state->getValue(['platforms', $key])) {
        $this->setConfigurationValue('platforms-' . $key, TRUE);
      }
      else {
        $this->setConfigurationValue('platforms-' . $key, FALSE);
      }
    }
    $this->setConfigurationValue('show_copy_link', $form_state->getValue('show_copy_link'));
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $current_url = Url::fromRoute('<current>')->setAbsolute()->toString();
    $current_title = \Drupal::service('title_resolver')->getTitle(\Drupal::request(), \Drupal::routeMatch()->getRouteObject());
    $config = $this->getConfiguration();
    // Build sharing URLs only for enabled platforms.
    $share_urls = [];
    $allowed_platforms = [];
    foreach (self::AVAILABLE_PLATFORMS as $key => $label) {
      if ($config['platforms-' . $key]) {
        $allowed_platforms[] = $key;
      }
    }
    $allowed_platforms = $allowed_platforms;
    foreach ($allowed_platforms as $platform) {
      switch ($platform) {
        case 'facebook':
          $share_urls['facebook'] = 'https://www.facebook.com/sharer/sharer.php?u=' . urlencode($current_url);
          break;

        case 'twitter':
          $share_urls['twitter'] = 'https://twitter.com/intent/tweet?url=' . urlencode($current_url) . '&text=' . urlencode($current_title);
          break;

        case 'linkedin':
          $share_urls['linkedin'] = 'https://www.linkedin.com/sharing/share-offsite/?url=' . urlencode($current_url);
          break;

        case 'whatsapp':
          $share_urls['whatsapp'] = 'https://wa.me/?text=' . urlencode($current_url);
          break;

        case 'telegram':
          $share_urls['telegram'] = 'https://t.me/share/url?url=' . urlencode($current_url) . '&text=' . urlencode($current_title);
          break;

        case 'pinterest':
          $share_urls['pinterest'] = 'https://pinterest.com/pin/create/button/?url=' . urlencode($current_url) . '&description=' . urlencode($current_title);
          break;

        case 'reddit':
          $share_urls['reddit'] = 'https://reddit.com/submit?url=' . urlencode($current_url) . '&title=' . urlencode($current_title);
          break;

        case 'tumblr':
          $share_urls['tumblr'] = 'https://www.tumblr.com/share/link?url=' . urlencode($current_url) . '&name=' . urlencode($current_title);
          break;

        case 'email':
          $share_urls['email'] = 'mailto:?subject=' . urlencode($current_title) . '&body=' . urlencode($current_url);
          break;
      }
    }

    return [
      '#theme' => 'simple_social_share_block',
      '#share_urls' => $share_urls,
      '#current_url' => $current_url,
      '#show_copy_link' => $config['show_copy_link'] ?? TRUE,
      '#attached' => [
        'library' => ['simple_social_share/social_share'],
      ],
    ];
  }

}
